import unittest

from django import forms
from django.template import Context, Template
from django.template.base import Parser, Variable
from django.test import RequestFactory

from django_genericfilters.templatetags.updateurl import (
    token_kwargs,
    token_value,
    update_query_string,
)
from django_genericfilters.templatetags.utils import is_checkbox


class TemplateTagTestCase(unittest.TestCase):
    def test_token_value(self):
        parser = Parser("")
        self.assertEqual(token_value('"A"', parser), "A")
        var = token_value("a", parser).var
        self.assertTrue(isinstance(var, Variable))

    def test_token_kwargs(self):
        parser = Parser("")
        self.assertEqual(token_kwargs([], parser), {})
        bits = ['a="A"']
        token_kwargs(bits, parser)
        self.assertEqual(bits, [])
        bits = ['a="A"', "invalid"]
        token_kwargs(bits, parser)
        self.assertEqual(bits, ["invalid"])

    def test_update_query_string(self):
        self.assertEqual(
            update_query_string("/foo/?bar=baz", {"bar": "updated"}),
            "/foo/?bar=updated",
        )
        self.assertEqual(
            update_query_string("/foo/?bar=baz", {"bar": "updated"}),
            "/foo/?bar=updated",
        )
        self.assertEqual(
            update_query_string("/foo/", {"bar": "created"}), "/foo/?bar=created"
        )

    def test_tag_update_query_string(self):
        template = Template(
            "{% load updateurl %}{% update_query_string with page=num_page %}"
        )
        self.assertEqual(
            template.render(
                Context(
                    {
                        "request": RequestFactory().get("/fake"),
                        "page": "page",
                        "num_page": 2,
                    }
                )
            ),
            "/fake?page=2",
        )

    def test_is_checkbox(self):
        class MockForm(forms.Form):
            a = forms.CharField()
            b = forms.BooleanField()

        form = MockForm()

        boundfield_a, boundfield_b = form.visible_fields()

        self.assertEqual("a", boundfield_a.name)
        self.assertFalse(is_checkbox(boundfield_a))

        self.assertEqual("b", boundfield_b.name)
        self.assertTrue(is_checkbox(boundfield_b))



import unittest

from django import forms

from django_genericfilters import forms as gf


class FieldTestCase(unittest.TestCase):
    def setUp(self):
        class Form(forms.Form):
            test = gf.ChoiceField(label="Test")

        self.form = Form()

    def test_filtered_choice_field_required(self):
        self.assertFalse(self.form.fields["test"].required)

    def test_filtered_choice_field_auto_choices(self):
        choices = self.form.fields["test"].choices

        self.assertEqual(choices[0][0], "yes")
        self.assertEqual(choices[0][1], "Test")

        self.assertEqual(choices[1][0], "no")
        self.assertEqual(choices[1][1], "No Test")



# coding=utf8
"""Test suite for django-generic-filters."""
from django.test import TestCase

from demoproject.compat import reverse


class HomeViewTestCase(TestCase):
    """Test homepage."""

    def test_get(self):
        """Homepage returns HTTP 200."""
        home_url = reverse("home")
        response = self.client.get(home_url)
        self.assertEqual(response.status_code, 200)



import unittest

from django import forms

from django_genericfilters import forms as gf


class FormTestCase(unittest.TestCase):
    def test_query_form_mixin(self):
        class Form(gf.QueryFormMixin, forms.Form):
            pass

        form = Form()

        self.assertTrue("query" in form.fields)

    def test_order_form_mixin(self):
        def get_order_by_choices():
            return (("last_name", "Last Name"), ("first_name", "First Name"))

        class Form(gf.OrderFormMixin, forms.Form):
            def get_order_by_choices(self):
                return get_order_by_choices()

        form = Form()

        self.assertTrue("order_by" in form.fields)
        self.assertTrue("order_reverse" in form.fields)

        self.assertEqual(form.fields["order_by"].choices, list(get_order_by_choices()))

    def test_get_order_by_choices_not_implemented(self):
        class Form(gf.OrderFormMixin):
            pass

        self.assertRaises(NotImplementedError, Form)


class FilteredFormTestCase(unittest.TestCase):

    # Define a form class for this test case
    class Form(gf.FilteredForm):
        def get_order_by_choices(self):
            return (("last_name", "Last Name"), ("first_name", "First Name"))

    def test_empty_form_bound(self):
        form = self.Form(data={}, initial={"order_by": "last_name"})
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data["order_by"], "last_name")

    def test_form_order_by_bound(self):
        form = self.Form(
            data={"order_by": "first_name", "order_reverse": 0},
            initial={"order_by": "last_name"},
        )
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data["order_by"], "first_name")

    def test_filtered_form(self):
        form = self.Form()

        self.assertEqual(
            str(type(form.fields["query"].widget)),
            "<class 'django.forms.widgets.HiddenInput'>",
        )

        self.assertEqual(
            str(type(form.fields["order_by"].widget)),
            "<class 'django.forms.widgets.HiddenInput'>",
        )

        self.assertEqual(
            str(type(form.fields["order_reverse"].widget)),
            "<class 'django.forms.widgets.HiddenInput'>",
        )



from django import forms
from django.utils.translation import gettext_lazy as _


class ChoiceField(forms.ChoiceField):
    def __init__(self, *args, **kwargs):

        kwargs.setdefault("required", False)

        if "choices" not in kwargs:
            kwargs["choices"] = (
                ("yes", kwargs.get("label", _("Yes"))),
                ("no", _("No %(label)s") % {"label": kwargs.get("label", "")}),
            )
        super(ChoiceField, self).__init__(*args, **kwargs)



from django.urls import include, path
from django.views.generic import TemplateView

home = TemplateView.as_view(template_name="home.html")


urlpatterns = [
    path(r"filter/", include("demoproject.filter.urls")),
    # An informative homepage.
    path(r"", home, name="home"),
]



"""Django settings for django-generic-filters demo project."""

from os import environ
from os.path import abspath, dirname, join

# Configure some relative directories.
demoproject_dir = dirname(abspath(__file__))
root_dir = dirname(demoproject_dir)
data_dir = join(root_dir, "var")


# Mandatory settings.
ROOT_URLCONF = "demoproject.urls"
WSGI_APPLICATION = "demoproject.wsgi.application"


# Database.
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "NAME": environ.get("PGDATABASE", "django_generic_filters")
        # Configure database using standard PG* environment variables
        # https://www.postgresql.org/docs/current/libpq-envars.html
    }
}

# Template.
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": True,
            "context_processors": (
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.template.context_processors.request",
                "django.contrib.messages.context_processors.messages",
            ),
        },
    },
]


# Required.
SECRET_KEY = "This is a secret made public on project's repository."

# Media and static files.
MEDIA_ROOT = join(data_dir, "media")
MEDIA_URL = "/media/"
STATIC_ROOT = join(data_dir, "static")
STATIC_URL = "/static/"


# Applications.
INSTALLED_APPS = (
    # Standard Django applications.
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # The actual django-generic-filters demo.
    "django_genericfilters",
    "demoproject",
    "demoproject.filter",
)


# Default middlewares. You may alter the list later.
MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
]

# Development configuration.
DEBUG = True

USE_TZ = True



from django.utils.translation import gettext_lazy as _

from django_genericfilters import forms as gf


class UserListForm(gf.FilteredForm):
    is_active = gf.ChoiceField(
        label=_("Status"), choices=(("yes", _("Active")), ("no", _("Unactive")))
    )

    is_staff = gf.ChoiceField(label=_("Staff"))

    is_superuser = gf.ChoiceField(label=_("Superuser"))

    def get_order_by_choices(self):
        return [
            ("date_joined", _("date joined")),
            ("last_login", _("last login")),
            ("last_name", _("Name")),
        ]



import urllib

import factory
from django import forms
from django.db import models
from django.http import QueryDict
from django.test import RequestFactory, TestCase
from django.utils.datastructures import MultiValueDict

from django_genericfilters import views
from django_genericfilters.forms import FilteredForm


def setup_view(view, request, *args, **kwargs):
    """Mimic as_view() returned callable, but returns view instance.

    args and kwargs are the same you would pass to ``reverse()``

    See also: https://code.djangoproject.com/ticket/20456

    """

    view.request = request
    view.args = args
    view.kwargs = kwargs

    return view


class People(models.Model):
    """
    define a parent model
    """

    name = models.CharField(max_length=250)


class Status(models.Model):
    """
    define a dummy status model
    """

    name = models.CharField(max_length=250)


class Something(models.Model):
    """
    Define a dummy model for this test case
    """

    people = models.ForeignKey(People, on_delete=models.CASCADE)
    city = models.CharField(max_length=250)
    country = models.CharField(max_length=250)
    organization = models.CharField(max_length=250)
    status = models.ForeignKey(Status, null=True, on_delete=models.CASCADE)


class StatusFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("word")

    class Meta:
        model = Status


class PeopleFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("name")

    class Meta:
        model = People


class SomethingFactory(factory.django.DjangoModelFactory):
    people = factory.SubFactory(PeopleFactory)
    status = factory.SubFactory(StatusFactory)
    city = factory.Faker("city")
    country = factory.Faker("country")
    organization = factory.Faker("company")

    class Meta:
        model = Something


class FilteredViewTestCase(TestCase):
    class Form(FilteredForm):

        city = forms.ChoiceField(
            label="city", required=False, choices=(("N", "Nantes"), ("P", "Paris"))
        )

        country = forms.ChoiceField(
            label="country",
            required=False,
            choices=(("F", "France"), ("P", "Portugal")),
        )

        people = forms.ChoiceField(
            label="people", required=False, choices=(("S", "Some"), ("A", "Any"))
        )

        organization = forms.MultipleChoiceField(
            label="organization",
            required=False,
            choices=(("A", "A Team"), ("B", "B Team"), ("C", "C Team")),
        )

        parent = forms.ModelChoiceField(
            queryset=People.objects.all(), label="parent", required=False
        )

        status = forms.ModelMultipleChoiceField(
            label="status", required=False, queryset=Status.objects.all()
        )

        def get_order_by_choices(self):
            return (("city", "City"), ("country", "Country"))

    @classmethod
    def setUpTestData(cls):
        SomethingFactory.create_batch(20)

    def test_default_order_fallback_form_valid(self):
        """Queryset is unordered if no default_order or data (valid form)."""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(model=Something, form_class=self.Form),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_order_fallback_form_invalid(self):
        """Queryset is unordered if no default_order or data (invalid form)."""
        data = {"city": "fake"}
        view = setup_view(
            views.FilteredListView(model=Something, form_class=self.Form),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_invalid(view.form)
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_order_fallback_form_empty(self):
        """Queryset is unordered if no default_order or data (empty form)."""
        request = RequestFactory().get("/fake")
        view = setup_view(
            views.FilteredListView(model=Something, form_class=self.Form), request
        )

        queryset = view.form_empty()
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_filter(self):
        """Test the default filter"""
        request = RequestFactory().get("/fake")
        view = setup_view(
            views.FilteredListView(
                model=Something,
                form_class=self.Form,
                default_filter={"is_active": "1", "page": "1"},
            ),
            request,
        )

        query_filter = urllib.parse.urlencode({"is_active": "1", "page": "1"})
        get_filter = view.get_form_kwargs()
        self.assertEqual(get_filter["data"], QueryDict(query_filter))

    def test_default_filter_submit(self):
        """Test the default filter submit"""
        data = {"city": "N"}
        request = RequestFactory().get("/fake", data)
        view = setup_view(
            views.FilteredListView(
                model=Something,
                form_class=self.Form,
                default_filter={"is_active": "1", "page": "1"},
            ),
            request,
        )

        query_filter = urllib.parse.urlencode(
            {"is_active": "1", "page": "1", "city": "N"}
        )
        get_filter = view.get_form_kwargs()
        self.assertEqual(get_filter["data"], QueryDict(query_filter))

    def test_default_order_form_valid(self):
        """Queryset is ordered by default_order when no order_by in request."""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(
                model=Something, form_class=self.Form, default_order="city"
            ),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ["city"])

    def test_default_order_form_invalid(self):
        """Queryset is ordered by default_order when no order_by in request
        and form is invalid."""
        data = {"city": "fake"}
        view = setup_view(
            views.FilteredListView(
                model=Something, form_class=self.Form, default_order="city"
            ),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_invalid(view.form)
        self.assertEqual(list(queryset.query.order_by), ["city"])

    def test_default_order_form_empty(self):
        """Queryset is ordered by default_order when no order_by in request."""
        request = RequestFactory().get("/fake")
        view = setup_view(
            views.FilteredListView(
                model=Something, form_class=self.Form, default_order="city"
            ),
            request,
        )
        queryset = view.form_empty()
        self.assertEqual(list(queryset.query.order_by), ["city"])

    def test_default_order_reverse(self):
        """To test order reverse"""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(
                model=Something, form_class=self.Form, default_order="-city"
            ),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ["-city"])

    def test_default_order_in_request(self):
        """Test with order_by in data."""
        data = {"city": "N", "order_by": "city"}
        view = setup_view(
            views.FilteredListView(
                model=Something, form_class=self.Form, default_order="-city"
            ),
            RequestFactory().get("/fake", data),
        )

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ["city"])

    def test_filtered_list_view(self):
        a = views.FilteredListView(
            filter_fields=["city"], form_class=self.Form, model=Something
        )

        b = views.FilteredListView(
            filter_fields=["city", "people"],
            qs_filter_fields={"people__name": "people"},
            form_class=self.Form,
            model=Something,
        )
        setattr(
            a,
            "request",
            type("obj", (object,), {"method": "GET", "GET": {"city": "N"}}),
        )

        setattr(
            b,
            "request",
            type("obj", (object,), {"method": "GET", "GET": {"people": "S"}}),
        )

        self.assertEqual({"city": "city"}, a.get_qs_filters())
        a.form.is_valid()
        self.assertIn(
            '"django_genericfilters_something"."city" = N',
            a.form_valid(a.form).query.__str__(),
        )

        self.assertEqual({"people__name": "people"}, b.get_qs_filters())
        b.form.is_valid()
        self.assertIn(
            '"django_genericfilters_people"."name" = S',
            b.form_valid(b.form).query.__str__(),
        )

    def test_filtered_list_view__none(self):
        """
        FIXED : None value understood as "IS NULL" filter instead of being ignored.
        """
        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people__name": "people"},
            form_class=self.Form,
            model=Something,
        )

        data = {"city": "None", "people": "S"}
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        self.assertIn(
            '"django_genericfilters_people"."name" = S',
            str(view.form_valid(view.form).query),
        )

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people__name": "people"},
            form_class=self.Form,
            model=Something,
        )

        data = {"city": "N", "people": "None"}
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        self.assertIn(
            '"django_genericfilters_something"."city" = N',
            str(view.form_valid(view.form).query),
        )

    def test_filtered_list_view__multiplechoice(self):
        """
        FIXED : filtered fields has HiddenWidget widgets that cannot handle
                multiple values. Use Field.hidden_widget instead.
        """
        view = views.FilteredListView(
            filter_fields=["organization"], form_class=self.Form, model=Something
        )

        data = MultiValueDict({"organization": ["A"]})
        setup_view(view, RequestFactory().get("/fake", data))

        self.assertTrue(view.form.is_valid(), view.form.errors)
        self.assertIn(
            '"django_genericfilters_something"."organization" IN (A)',
            str(view.form_valid(view.form).query),
        )

        view = views.FilteredListView(
            filter_fields=["organization"], form_class=self.Form, model=Something
        )

        data = MultiValueDict({"organization": ["A", "C"]})
        setup_view(view, RequestFactory().get("/fake", data))
        self.assertTrue(view.form.is_valid())

    def test_filtered_list_view__multiplechoice__qs_filter_field(self):
        """
        FIXED : When using qs_filter_field, the behaviour changes because
                the HiddenWidget trick only works with filter_field
                attribute. But it compares a list with EQUAL operator
                instead of IN.
        """
        people = People.objects.create(name="fake")

        Something.objects.create(organization="A", people=people)
        Something.objects.create(organization="C", people=people)

        view = views.FilteredListView(
            qs_filter_fields={"organization": "organization"},
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"organization": ["A"]})
        setup_view(view, RequestFactory().get("/fake", data))

        self.assertTrue(view.form.is_valid(), view.form.errors)
        self.assertIn(
            '"django_genericfilters_something"."organization" IN (A)',
            str(view.form_valid(view.form).query),
        )
        self.assertEqual(1, view.form_valid(view.form).count())

        view = views.FilteredListView(
            qs_filter_fields={"organization": "organization"},
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"organization": ["A", "C"]})
        setup_view(view, RequestFactory().get("/fake", data))
        self.assertTrue(view.form.is_valid())
        self.assertEqual(2, view.form_valid(view.form).count())

    def test_filtered_list_view__modelchoice(self):
        peopleA = People.objects.create(name="fakeA")

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people": "parent"},
            form_class=self.Form,
            model=Something,
        )

        data = {"parent": peopleA.pk}
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        self.assertIn(
            '"django_genericfilters_something"."people_id" = %s' % (peopleA.pk,),
            str(view.form_valid(view.form).query),
        )

    def test_filtered_list_view__modelchoice__empty_queryset(self):
        """
        FIXED : Empty queryset in ModelChoiceField add "IS NULL" filters
                instead of ignore it.
        """
        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people": "parent"},
            form_class=self.Form,
            model=Something,
        )

        data = {"city": "N", "parent": 1}
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        self.assertIn(
            '"django_genericfilters_something"."city" = N',
            str(view.form_valid(view.form).query),
        )

    def test_filtered_list_view__modelchoice__none(self):
        """
        FIXED : Empty queryset in ModelChoiceField add "IS NULL" filters
                instead of ignore it.
        """
        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people": "parent"},
            form_class=self.Form,
            model=Something,
        )

        data = {"city": "N", "parent": "None"}
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        self.assertIn(
            '"django_genericfilters_something"."city" = N',
            str(view.form_valid(view.form).query),
        )

    def test_filtered_list_view__modelchoice_unhashable(self):
        """
        On Django3, the ModelChoiceIteratorValue is not hashable
        causing an error on get_filters() method
        """
        peopleA = People.objects.create(name="fakeA")

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "people": "parent"},
            filter_fields=["parent"],
            form_class=self.Form,
            model=Something,
        )
        data = {"parent": peopleA.pk}
        setup_view(view, RequestFactory().get("/fake", data))
        assert view.form.is_valid()
        filters = view.get_filters()
        # assert peopleA is selected on the filter choices
        assert peopleA.pk == next(
            getattr(f.value, "value", f.value)
            for f in filters[0].choices
            if f.is_selected
        )

    def test_filtered_list_view__multiplemodelchoice(self):
        """
        FIXED : filtered fields has HiddenWidget widgets that cannot handle
                multiple values. Use Field.hidden_widget instead.
        """
        stateA = Status.objects.create(name="stateA")
        stateB = Status.objects.create(name="stateB")
        stateC = Status.objects.create(name="stateC")
        people = People.objects.create(name="fake")

        A = Something.objects.create(organization="A", people=people, status=stateA)
        B = Something.objects.create(organization="B", people=people, status=stateB)
        C = Something.objects.create(organization="C", people=people, status=stateB)
        Something.objects.create(organization="D", people=people, status=stateC)

        view = views.FilteredListView(
            filter_fields=["city", "status"],
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"status": [stateA.pk]})
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()

        self.assertIsInstance(
            view.form.fields["status"].widget, forms.MultipleHiddenInput
        )

        queryset = view.form_valid(view.form)
        self.assertIn("IN (%s)" % stateA.pk, str(queryset.query))
        self.assertEqual([A], list(queryset.all()))

        view = views.FilteredListView(
            filter_fields=["city", "status"],
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"status": [stateA.pk, stateB.pk]})
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()

        queryset = view.form_valid(view.form)
        self.assertEqual([A, B, C], list(queryset.all()))

    def test_filtered_list_view__multiplemodelchoice__qs_filter_field(self):
        stateA = Status.objects.create(name="stateA")
        stateB = Status.objects.create(name="stateB")
        stateC = Status.objects.create(name="stateC")
        people = People.objects.create(name="fake")

        A = Something.objects.create(organization="A", people=people, status=stateA)
        B = Something.objects.create(organization="B", people=people, status=stateB)
        C = Something.objects.create(organization="C", people=people, status=stateB)
        Something.objects.create(organization="D", people=people, status=stateC)

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "status": "status"},
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"status": [stateA.pk]})
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()

        self.assertIsInstance(view.form.fields["status"].widget, forms.SelectMultiple)

        queryset = view.form_valid(view.form)
        self.assertIn("IN (%s)" % stateA.pk, str(queryset.query))
        self.assertEqual([A], list(queryset))

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "status": "status"},
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"status": [stateA.pk, stateB.pk]})
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()

        queryset = view.form_valid(view.form)
        self.assertEqual([A, B, C], list(queryset))

    def test_filtered_list_view__multiplemodelchoice__invalid_id(self):
        """
        FIXED : Invalid id in MultipleModelChoiceField generate a None
                value and add "IS NULL" filter instead of ignore it.
        """
        Status.objects.create(name="stateA")
        Status.objects.create(name="stateB")

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "status": "status"},
            form_class=self.Form,
            model=Something,
        )

        data = MultiValueDict({"status": [1001]})
        setup_view(view, RequestFactory().get("/fake", data))
        view.form.is_valid()
        # no filter at all
        self.assertNotIn("WHERE", str(view.form_valid(view.form).query))

    def test_filtered_list_view__multiplemodelchoice__none(self):
        """
        FIXED : Empty queryset in MultipleModelChoiceField is added as
                subrequest in filter and raises an sql error instead of
                ignore it.
        """
        people = People.objects.create(name="fake")

        statusA = Status.objects.create(name="stateA")
        statusB = Status.objects.create(name="stateB")

        Something.objects.create(organization="A", people=people, status=statusA)
        Something.objects.create(organization="C", people=people, status=statusB)

        view = views.FilteredListView(
            qs_filter_fields={"city": "city", "status": "status"},
            form_class=self.Form,
            model=Something,
        )

        setup_view(view, RequestFactory().get("/fake", {}))
        view.form.is_valid()
        # no filter at all
        self.assertNotIn("WHERE", str(view.form_valid(view.form).query))

    def test_is_form_submitted_method(self):
        """Is form submitted return True when the request method is GET."""
        request = RequestFactory().get("/fake", {"foo": "bar"})
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is True

        request = RequestFactory().post("/fake", {"foo": "bar"})
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is False

    def test_is_form_submitted_no_args(self):
        """Is form submitted return False when the queryset is empty."""
        request = RequestFactory().get("/fake")
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is False



#  Based on: http://www.djangosnippets.org/snippets/73/
#
#  Modified by Sean Reifschneider to be smarter about surrounding page
#  link context.  For usage documentation see:
#
#     http://www.tummy.com/Community/Articles/django-pagination/

from django import template

register = template.Library()


def paginator(context, adjacent_pages=1):
    """
    To be used in conjunction with the object_list generic view.

    Adds pagination context variables for use in displaying first, adjacent and
    last page links in addition to those created by the object_list generic
    view.

    """
    if "page_obj" in context:
        page_obj = context["page_obj"]
        paginator = context["paginator"]
        startPage = max(page_obj.number - adjacent_pages, 1)
        if startPage <= 3:
            startPage = 1
        endPage = page_obj.number + adjacent_pages + 1
        if endPage >= paginator.num_pages - 1:
            endPage = paginator.num_pages + 1
        page_numbers = [
            n for n in range(startPage, endPage) if n > 0 and n <= paginator.num_pages
        ]

        context["page_numbers"] = page_numbers
        context["show_first"] = 1 not in page_numbers
        context["show_last"] = paginator.num_pages not in page_numbers
        return context
    else:
        return {}


register.inclusion_tag("snippets/pagination.html", takes_context=True)(paginator)



from django.contrib.auth.models import User

from demoproject.filter.forms import UserListForm
from django_genericfilters.views import FilteredListView


class UserListView(FilteredListView):
    # Normal ListView options
    template_name = "user/user_list.html"
    paginate_by = 10
    context_object_name = "users"
    model = User

    # FilteredListView options
    form_class = UserListForm
    search_fields = ["first_name", "last_name", "username", "email"]
    filter_fields = ["is_active", "is_staff", "is_superuser"]
    default_order = "last_name"


user_list_view = UserListView.as_view()



from django.test import TestCase

from demoproject.compat import reverse


class FilteredListView(TestCase):
    fixtures = ["test_data.json"]

    def test_empty_get(self):
        url = reverse("user_filter_view")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_order_by_get(self):
        url = reverse("user_filter_view") + "?order_by=last_login"
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_query(self):
        url = reverse("user_filter_view") + "?query=doe"
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context["users"]), 1)
